unit MMJHint;
{TMMJHint is a component that changes the appearance of tiny hint window:
it changes hint's shape and background.

Happy programming
Mihaela Mihaljevic Jakic
mickj@hi.hinet.hr
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TShapeKind = (skEllipse,skCircle,skRectangle,skRoundRectangle,skText,skNotText);
  TBackgroundKind = (bkPicture,bkFill);
  TFillType =(GradUpDown, GradLeftRight, GradDiagonal1, GradDiagonal2);
  TMMJHint = class(TComponent)
  private
    { Private declarations }
    FFont:TFont;
    FTextColor :TColor;
    FEnabled : Boolean;
    FHasFrame:Boolean;
    FShapeKind : TShapeKind;
    FStartColor : TColor;
    FEndColor : TColor;
    FFrameColor : TColor;
    FBackgroundKind : TBackgroundKind;
    FFillMode : TFillType;
    FPicture: TPicture;
    procedure SetStartColor(Value : TColor);
    procedure SetEndColor(Value : TColor);
    procedure SetFrameColor(Value : TColor);
    procedure SetShapeKind(Value : TShapeKind);
    procedure SetFont(Value:TFont);
    procedure SetEnabled(Value:Boolean);
    procedure SetHasFrame(Value:Boolean);
    procedure SetTextColor(Value:TColor);
    procedure SetPicture(Value: TPicture);
    procedure SetBackgroundKind(Value:TBackgroundKind);
    procedure SetFillMode(Value:TFillType);
  protected
    { Protected declarations }
  published
    { Published declarations }
    property Font:TFont
       read FFont
       write SetFont;
    property TextColor : Tcolor
       read FTextColor
       write SetTextColor;
    property Enabled : Boolean
       read FEnabled
       write SetEnabled;
    property HasFrame : Boolean
       read FHasFrame
       write SetHasFrame;
   property ShapeKind : TShapeKind
      read FShapeKind
      write SetShapeKind;
    property StartColor : TColor
       read FStartColor
       write SetStartColor;
    property EndColor : TColor
       read FEndColor
       write SetEndColor;
   property FrameColor : TColor
      read FFrameColor
      write SetFrameColor;
    property FillMode : TFillType
       read FFillMode
       write SetFillMode;
    property BackgroundKind : TBackgroundKind
       read FBackgroundKind
       write SetBackgroundKind;
    property Picture: TPicture
       read FPicture
       write SetPicture;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
   // destructor Destroy; override;
  end;

  TMMJHintWindow = class(THintWindow)
  private
    { Private declarations }
    BodyRegion : hRgn;
    FrameRegion : hRgn;
    ShadowRegion : hRgn;
    FHintFont : TFont;
    FTextColor:TColor;
    FMMJHint :TMMJHint;
    Str:string;
    function GetMMJHint:TMMJHint;
    procedure SetTextColor(Value:TColor);
    function IsTrueTypeFont(FFont:TFont) : Boolean;
    procedure DeleteRegions;
    procedure SetFont(Value:TFont);
    procedure PaintCanvas(Width,Height:integer);
    procedure GradientUpDown(FromRGB, ToRGB: TColor; wid,Hei:integer);
    procedure GradientLeftRight(FromRGB, ToRGB: TColor;Width,Height:integer);
    procedure GradientDiagonal1(FromRGB, ToRGB: TColor;Width,Height:integer);
    procedure GradientDiagonal2(FromRGB, ToRGB: TColor;Width,Height:integer);
  protected
    { Protected declarations }
    procedure Paint; override;
    procedure CreateParams(var Params:TCreateParams);override;
    procedure ActivateHint(Rect:TRect;const AHint:string);override;
    function CalcSize(str:string):TSize;
    property HintFont : TFont
       read FHintFont
       write SetFont ;
    property TextColor : TColor
       read FTextColor
       write SetTextColor default clNavy;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Mihaela''s', [TMMJHint]);
end;
//Get the wrapper componenet
function TMMJHintWindow.GetMMJHint:TMMJHint;
var
 i:Integer;
begin
 Result:=nil;
 for i:=0 to Application.MainForm.ComponentCount-1 do
  if Application.MainForm.Components[i] is TMMJHint then begin
   Result:=TMMJHint(Application.MainForm.Components[i]);
   Exit;
  end;
end;

constructor TMMJHintWindow.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   FMMJHint:=GetMMJHint;
   FTextColor:=FMMJHint.TextColor;
   FHintFont:=TFont.Create;
   FHintFont:=FMMJHint.Font;
   if (HintFont=nil) or not IsTrueTypeFont(FHintFont) then begin
      FHintFont.Name:='Arial';
      FHintFont.Style:=[fsBold];
      FHintFont.Size:=16;
   end;
      Hint:='TMMJHint';
      //CalcSize(Hint);
  // FEnabled:=False;
end;

destructor TMMJHintWindow.Destroy;
begin
   DeleteRegions;
   SetWindowRgn(Handle,0,True);
   FHintFont.Free;
   inherited Destroy;
end;

procedure TMMJHintWindow.CreateParams(var Params:TCreateParams);
begin
   inherited CreateParams(Params);
   Params.Style:=Params.Style and not ws_Border;
  // Params.WindowClass.Style := Params.WindowClass.Style OR CS_SAVEBITS;
end;

procedure TMMJHintWindow.GradientUpDown(FromRGB, ToRGB: TColor; wid,Hei:integer);
var
   { from RGB values }
   RGBFrom : array[0..2] of Byte;
   { difference of from/to RGB values }
   RGBDiff : array[0..2] of integer;
   { color band rectangular coordinates }
   ColorBand : TRect;
   { color band index }
   i : Integer;
   { a color band's components }
   r, g, b : Byte;
   //wrect :TRect;
begin
   { Extract from RGB values. }
   RGBFrom[0] := GetRValue(ColorToRGB(FromRGB));
   RGBFrom[1] := GetGValue(ColorToRGB(FromRGB));
   RGBFrom[2] := GetBValue(ColorToRGB(FromRGB));
   { Calculate difference of from and to RGB values.}
   RGBDiff[0] := GetRValue(ColorToRGB(ToRGB)) - RGBFrom[0];
   RGBDiff[1] := GetGValue(ColorToRGB(ToRGB)) - RGBFrom[1];
   RGBDiff[2] := GetBValue(ColorToRGB(ToRGB)) - RGBFrom[2];
   { Set pen sytle and mode. }
   Canvas.Pen.Style := psSolid;
   Canvas.Pen.Mode := pmCopy;
   { Set color band's left and right coordinates. }
   ColorBand.Left := 0;
   ColorBand.Right := Wid;
   for i := 0 to $FF do begin
      { Calculate color band's top and bottom coordinates. }
      ColorBand.Top := MulDiv(i, Hei, $100);
      ColorBand.Bottom := MulDiv(i + 1, Hei, $100);
      { Calculate color band color. }
      r := RGBFrom[0] + MulDiv(i, RGBDiff[0], $FF);
      g := RGBFrom[1] + MulDiv(i, RGBDiff[1], $FF);
      b := RGBFrom[2] + MulDiv(i, RGBDiff[2], $FF);
      { Select brush and paint color band. }
      Canvas.Brush.Color := RGB(r, g, b);
      Canvas.FillRect(ColorBand);
   end;
end;

procedure TMMJHintWindow.GradientLeftRight(FromRGB, ToRGB: TColor;Width,Height:integer);
var
   { from RGB values }
   RGBFrom : array[0..2] of Byte;
   { difference of from/to RGB values }
   RGBDiff : array[0..2] of integer;
   { color band rectangular coordinates }
   ColorBand : TRect;
   { color band index }
   i : Integer;
   { a color band's components }
   r, g, b : Byte;
begin
   { Extract from RGB values. }
   RGBFrom[0] := GetRValue(ColorToRGB(FromRGB));
   RGBFrom[1] := GetGValue(ColorToRGB(FromRGB));
   RGBFrom[2] := GetBValue(ColorToRGB(FromRGB));
   RGBDiff[0] := GetRValue(ColorToRGB(ToRGB)) - RGBFrom[0];
   RGBDiff[1] := GetGValue(ColorToRGB(ToRGB)) - RGBFrom[1];
   RGBDiff[2] := GetBValue(ColorToRGB(ToRGB)) - RGBFrom[2];
   Canvas.Pen.Style := psSolid;
   Canvas.Pen.Mode := pmCopy;
   ColorBand.Top := 0;
   ColorBand.Bottom := Height;
  for i := 0 to Width do begin
     ColorBand.Left := i;
      ColorBand.Right := i+1;
      R := GetRValue(FromRGB)+i*(GetRValue(ToRGB)-GetRValue(FromRGB))div Width;
      G := GetGValue(FromRGB)+i*(GetGValue(ToRGB)-GetGValue(FromRGB))div Width;
      B := GetBValue(FromRGB)+i*(GetBValue(ToRGB)-GetBValue(FromRGB))div Width;
      Canvas.Brush.Color := RGB(r, g, b);
      Canvas.FillRect(ColorBand);
   end;
end;


procedure TMMJHintWindow.GradientDiagonal1(FromRGB, ToRGB: TColor;Width,Height:integer);
var
   val,i : integer;
   R, G, B: Byte;
begin
    val := Width+Height;
    for i := 0 to val do begin
      R := GetRValue(FromRGB)+i*(GetRValue(ToRgb)-GetRValue(FromRGB))div val;
      G := GetGValue(FromRGB)+i*(GetGValue(ToRgb)-GetGValue(FromRGB))div val;
      B := GetBValue(FromRGB)+i*(GetBValue(ToRgb)-GetBValue(FromRGB))div val;
      Canvas.Pen.Color := RGB(R, G, B);
      MoveToEx(Canvas.Handle, i, 0, nil); LineTo(Canvas.Handle, 0, i);
    end;
end;


procedure TMMJHintWindow.GradientDiagonal2(FromRGB, ToRGB: TColor;Width,Height:integer);
var
   val,i : integer;
   R, G, B: Byte;
begin

    val := Width+Height;
    for i := 0 to val do begin
      R := GetRValue(FromRGB)+i*(GetRValue(ToRGB)-GetRValue(FromRGB))div val;
      G := GetGValue(FromRGB)+i*(GetGValue(ToRGB)-GetGValue(FromRGB))div val;
      B := GetBValue(FromRGB)+i*(GetBValue(ToRGB)-GetBValue(FromRGB))div val;
      Canvas.Pen.Color := RGB(R, G, B);
      MoveToEx(Canvas.Handle, 0, Height-i, nil); LineTo(Canvas.Handle, i, Height);
    end;
end;

procedure TMMJHintWindow.PaintCanvas(Width,Height:integer);
var
   fw,fh,bw,bh,i,j,x,y:integer;
begin
   //Painting gradient
   if FMMJHint.FBackgroundKind = bkPicture then begin
      if FMMJHint.FPicture.Bitmap.Empty then begin
         FMMJHint.FBackgroundKind:=bkFill;
         Exit;
      end;
         fw:=Width;
         fh:=Height;
         bw:=FMMJHint.FPicture.Graphic.Width;
         bh:=FMMJHint.FPicture.Graphic.Height;
         x:=fw div bw;
         y:=fh div bh;
       for i:= 0 to x do begin
          for j:= 0 to y do begin
             Canvas.Draw(i*bw,j*bh,FMMJHint.FPicture.Graphic);
          end;
       end;
   end
   else if FMMJHint.FBackgroundKind = bkFill then begin
      case FMMJHint.FFillMode of
         GradUpDown    :GradientUpDown(FMMJHint.FStartColor,FMMJHint.FEndColor,Width,Height);
         GradLeftRight :GradientLeftRight(FMMJHint.FStartColor,FMMJHint.FEndColor,Width,Height);
         GradDiagonal1 :GradientDiagonal1(FMMJHint.FStartColor,FMMJHint.FEndColor,Width,Height);
         GradDiagonal2 :GradientDiagonal2(FMMJHint.FStartColor,FMMJHint.FEndColor,Width,Height);
      end;
   end;
end;

procedure TMMJHintWindow.Paint;
begin
   //Filling main window
   //Canvas.Brush.Color:=clYellow; //Main window color
   //FillRgn(Canvas.Handle,BodyRegion,Canvas.Brush.Handle);
   if FMMJHint.ShapeKind = skEllipse then begin
      GradientUpDown(clGray,clDKGray,Width,Height);
      Canvas.Brush.Color:=FMMJHint.TextColor;//TextColor
      FillRgn(Canvas.Handle,ShadowRegion,Canvas.Brush.Handle);
      //Fills Body region
      PaintCanvas(Width,Height);
      SetBkMode(Canvas.Handle, TRANSPARENT);
      Canvas.Font.Color:=FMMJHint.TextColor;
      Canvas.TextOut(8,5,Str);
      //Fills RgameRegion
      if FMMJHint.HasFrame then begin
         Canvas.Brush.Color:=FMMJHint.FrameColor;
         FrameRgn(Canvas.Handle,FrameRegion,Canvas.Brush.Handle,1,1);
      end;
   end;

   if FMMJHint.ShapeKind = skCircle then begin
      GradientUpDown(clGray,clDKGray,Width,Height);
      Canvas.Brush.Color:=FMMJHint.TextColor;//TextColor
      FillRgn(Canvas.Handle,ShadowRegion,Canvas.Brush.Handle);
      //Fills Body region
      PaintCanvas(Width,Height);
      SetBkMode(Canvas.Handle, TRANSPARENT);
      Canvas.Font.Color:=FMMJHint.TextColor;
      Canvas.TextOut(8,5,Str);
      //Fills RgameRegion
      if FMMJHint.HasFrame then begin
         Canvas.Brush.Color:=FMMJHint.FrameColor;
         FrameRgn(Canvas.Handle,FrameRegion,Canvas.Brush.Handle,1,1);
      end;
   end;

   if FMMJHint.ShapeKind = skCircle then begin
      GradientUpDown(clGray,clDKGray,Width,Height);
      Canvas.Brush.Color:=FMMJHint.TextColor;//TextColor
      FillRgn(Canvas.Handle,ShadowRegion,Canvas.Brush.Handle);
      //Fills Body region
      PaintCanvas(Width,Height);
      SetBkMode(Canvas.Handle, TRANSPARENT);
      Canvas.Font.Color:=FMMJHint.TextColor;
      Canvas.TextOut(8,5,Str);
      //Fills RgameRegion
      if FMMJHint.HasFrame then begin
         Canvas.Brush.Color:=FMMJHint.FrameColor;
         FrameRgn(Canvas.Handle,FrameRegion,Canvas.Brush.Handle,1,1);
      end;
   end;

   if (FMMJHint.ShapeKind = skRoundRectangle) or
      (FMMJHint.ShapeKind = skRectangle)then begin
      GradientUpDown(clGray,clDKGray,Width,Height);
      Canvas.Brush.Color:=FMMJHint.TextColor;//TextColor
      FillRgn(Canvas.Handle,ShadowRegion,Canvas.Brush.Handle);
      //Fills Body region
      PaintCanvas(Width,Height);
      SetBkMode(Canvas.Handle, TRANSPARENT);
      Canvas.Font.Color:=FMMJHint.TextColor;
      Canvas.TextOut(8,5,Str);
      //Fills RgameRegion
      if FMMJHint.HasFrame then begin
         Canvas.Brush.Color:=FMMJHint.FrameColor;
         FrameRgn(Canvas.Handle,FrameRegion,Canvas.Brush.Handle,1,1);
      end;
   end;

   if FMMJHint.ShapeKind = skText then begin
      Canvas.Brush.Color:=clBlack;//ShadowColor
      FillRgn(Canvas.Handle,ShadowRegion,Canvas.Brush.Handle);
      PaintCanvas(Width,Height);
      //Canvas.Brush.Color:=FMMJHint.TextColor;//TextColor
      //FillRgn(Canvas.Handle,ShadowRegion,Canvas.Brush.Handle);
      if FMMJHint.HasFrame then begin
         Canvas.Brush.Color:=FMMJHint.FrameColor;
         FrameRgn(Canvas.Handle,FrameRegion,Canvas.Brush.Handle,1,1);
      end;
   end;

   if FMMJHint.ShapeKind = skNotText then begin
      Canvas.Brush.Color:=clBlack;//ShadowColor
      FillRgn(Canvas.Handle,ShadowRegion,Canvas.Brush.Handle);
      PaintCanvas(Width,Height);
     // Canvas.Brush.Color:=FMMJHint.TextColor;//TextColor
    //  FillRgn(Canvas.Handle,ShadowRegion,Canvas.Brush.Handle);
     // GradientUpDown(FMMJHint.StartColor,FMMJHint.EndColor,Width,Height);
     if FMMJHint.HasFrame then begin
         Canvas.Brush.Color:=FMMJHint.FrameColor;
         FrameRgn(Canvas.Handle,FrameRegion,Canvas.Brush.Handle,1,1);
     end;
   end;
end;

procedure TMMJHintWindow.DeleteRegions;
begin
   if BodyRegion <> 0 then begin
      DeleteObject(BodyRegion);
      BodyRegion:=0;
   end;
   if FrameRegion <> 0 then begin
      DeleteObject(FrameRegion);
      FrameRegion:=0;
   end;
   if ShadowRegion <> 0 then begin
      DeleteObject(ShadowRegion);
      ShadowRegion:=0;
   end;
end;

procedure TMMJHintWindow.ActivateHint(Rect:TRect;const AHint:string);
var
   RegionData: Pointer;
   RgnDataSize: DWORD;
   sz:TSize;
   whDiff,i : integer;
begin
   //deleting regions
   Str:=AHint;
   Rect.Right:=Rect.Right + 6;
   Rect.Bottom:=Rect.Bottom + 4;
   DeleteRegions;
   CalcSize(AHint);
 //Width := Canvas.TextWidth(Caption) + 6;
 // Height := Canvas.TextHeight(Caption) + 4;
   if FMMJHint.ShapeKind = skEllipse then begin
      Canvas.Font.Assign(FMMJHint.Font);
      sz:=CalcSize(Ahint);
      Width:=sz.cx ;
      Height:=sz.cy ;
      Inc(Rect.Right, 6);
      Inc(Rect.Bottom, 6);
      UpdateBoundsRect(Rect);
      BodyRegion:=CreateEllipticRgn(0,0,Width,Height);
      FrameRegion:=CreateEllipticRgn(0,0,Width,Height);
      ShadowRegion:=CreateEllipticRgn(0,0,Width,Height);
      OffsetRgn(ShadowRegion,-2,2);
      CombineRgn(ShadowRegion,ShadowRegion,BodyRegion,RGN_DIFF );
      //Canvas.Font.Assign(FHintFont);
      Canvas.Font.Size:=FMMJHint.Font.Size-1;
     Repaint;
   end;

   if FMMJHint.ShapeKind = skRoundRectangle then begin
      Canvas.Font.Assign(FMMJHint.Font);
      sz:=CalcSize(Ahint);
      Width:=sz.cx ;
      Height:=sz.cy ;
      Inc(Rect.Right, 6);
      Inc(Rect.Bottom, 6);
      UpdateBoundsRect(Rect);
      BodyRegion:=CreateRoundRectRgn(0,0,Width,Height,25,25);
      FrameRegion:=CreateRoundRectRgn(0,0,Width,Height,25,25);
      ShadowRegion:=CreateRoundRectRgn(0,0,Width,Height,25,25);
      OffsetRgn(ShadowRegion,-2,2);
      CombineRgn(ShadowRegion,ShadowRegion,BodyRegion,RGN_DIFF );
      //Canvas.Font.Assign(FHintFont);
      Canvas.Font.Size:=FMMJHint.Font.Size-1;
     Repaint;
   end;

    if FMMJHint.ShapeKind = skRectangle then begin
      Canvas.Font.Assign(FMMJHint.Font);
      sz:=CalcSize(Ahint);
      Width:=sz.cx ;
      Height:=sz.cy ;
      Inc(Rect.Right, 6);
      Inc(Rect.Bottom, 6);
      UpdateBoundsRect(Rect);
      BodyRegion:=CreateRectRgn(0,0,Width,Height);
      FrameRegion:=CreateRectRgn(0,0,Width,Height);
      ShadowRegion:=CreateRectRgn(0,0,Width,Height);
      OffsetRgn(ShadowRegion,-2,2);
      CombineRgn(ShadowRegion,ShadowRegion,BodyRegion,RGN_DIFF );
      //Canvas.Font.Assign(FHintFont);
      Canvas.Font.Size:=FMMJHint.Font.Size-1;
     Repaint;
   end;

    if FMMJHint.ShapeKind = skCircle then begin
      Canvas.Font.Assign(FMMJHint.Font);
      sz:=CalcSize(Ahint);
      Width:=sz.cx ;
      Height:=sz.cy ;
      Inc(Rect.Right, 6);
      Inc(Rect.Bottom, 6 );
      UpdateBoundsRect(Rect);
      BodyRegion:=CreateEllipticRgn(0,0,Width div 2 + Width div 6,Height);
      FrameRegion:=CreateEllipticRgn(Width div 2-Width div 6,0,Width,Height);
      CombineRgn(BodyRegion,BodyRegion,FrameRegion,RGN_OR);
      OffsetRgn(ShadowRegion,-2,2);
      CombineRgn(ShadowRegion,ShadowRegion,BodyRegion,RGN_DIFF );

      DeleteObject(FrameRegion);
      FrameRegion:=0;
      
      RgnDataSize := GetRegionData(BodyRegion, 0, NIL);
      GetMem(RegionData, RgnDataSize);
      GetRegionData(BodyRegion, RgnDataSize, RegionData);
      FrameRegion := ExtCreateRegion(nil, RgnDataSize, TRgnData(RegionData^));
      FreeMem(RegionData, RgnDataSize);
      //Creating Shadow region
      RgnDataSize := GetRegionData(BodyRegion, 0, NIL);
      GetMem(RegionData, RgnDataSize);
      GetRegionData(BodyRegion, RgnDataSize, RegionData);
      ShadowRegion := ExtCreateRegion(nil, RgnDataSize, TRgnData(RegionData^));
      FreeMem(RegionData, RgnDataSize);
      //Canvas.Font.Assign(FHintFont);
      Canvas.Font.Size:=FMMJHint.Font.Size-1;
     Repaint;
   end;

   if FMMJHint.ShapeKind = skText then begin
      Canvas.Font.Assign(FHintFont);
      sz:=CalcSize(AHint);
      Width:=sz.cx + 6;
      Height:=sz.cy + 6;
      BeginPath(Canvas.Handle);
      SetBkMode(Canvas.Handle, TRANSPARENT);
      TextOut(Canvas.Handle, 0, 0,PChar(AHint), Length(AHint));
      EndPath(Canvas.Handle);
      BodyRegion := PathToRegion(Canvas.Handle);
      //creating Frame Region
      RgnDataSize := GetRegionData(BodyRegion, 0, NIL);
      GetMem(RegionData, RgnDataSize);
      GetRegionData(BodyRegion, RgnDataSize, RegionData);
      FrameRegion := ExtCreateRegion(nil, RgnDataSize, TRgnData(RegionData^));
      FreeMem(RegionData, RgnDataSize);
      //Creating Shadow region
      RgnDataSize := GetRegionData(BodyRegion, 0, NIL);
      GetMem(RegionData, RgnDataSize);
      GetRegionData(BodyRegion, RgnDataSize, RegionData);
      ShadowRegion := ExtCreateRegion(nil, RgnDataSize, TRgnData(RegionData^));
      FreeMem(RegionData, RgnDataSize);
      //Offset Shadow region
      OffsetRgn(ShadowRegion,-2,2);
      //CombineRgn(BodyRegion,BodyRegion,ShadowRegion,RGN_OR);
   end;
   if FMMJHint.ShapeKind = skNotText then begin
      Canvas.Font.Assign(FHintFont);
      sz:=CalcSize(AHint);
      Width:=sz.cx + 6;
      Height:=sz.cy + 6;
      BeginPath(Canvas.Handle);
      TextOut(Canvas.Handle, 0, 0,PChar(AHint), Length(AHint));
      EndPath(Canvas.Handle);
      BodyRegion := PathToRegion(Canvas.Handle);
      //creating Frame Region
      RgnDataSize := GetRegionData(BodyRegion, 0, NIL);
      GetMem(RegionData, RgnDataSize);
      GetRegionData(BodyRegion, RgnDataSize, RegionData);
      FrameRegion := ExtCreateRegion(nil, RgnDataSize, TRgnData(RegionData^));
      FreeMem(RegionData, RgnDataSize);
      //Creating Shadow region
      RgnDataSize := GetRegionData(BodyRegion, 0, NIL);
      GetMem(RegionData, RgnDataSize);
      GetRegionData(BodyRegion, RgnDataSize, RegionData);
      ShadowRegion := ExtCreateRegion(nil, RgnDataSize, TRgnData(RegionData^));
      FreeMem(RegionData, RgnDataSize);
      //Offset Shadow region
      //OffsetRgn(ShadowRegion,-2,2);
      CombineRgn(BodyRegion,BodyRegion,ShadowRegion,RGN_OR);
   end;

   SetWindowRgn(Handle,BodyRegion,True);
   inherited ActivateHint(Rect,AHint);
end;

function TMMJHintWindow.IsTrueTypeFont(FFont:TFont) : Boolean;
var
  FontInfo: TTextMetric;
  //FontInfo:lpTM;           // holds the font metric information
begin
   result:=False;
   Canvas.Font.Assign(FFont);
   Windows.GetTextMetrics(Canvas.Handle, FontInfo);
   with FontInfo do
      if ((tmPitchAndFamily and $0F) and TMPF_TRUETYPE) = TMPF_TRUETYPE then
         result:=True;
end;

function TMMJHintWindow.CalcSize(str:string):TSize;
var
   CurrentStringSize: TSize;
   TheString :PChar;
begin
   TheString:=PChar(str);
   GetTextExtentPoint32(Canvas.Handle, TheString,
                         Length(TheString), CurrentStringSize);
   result:=CurrentStringSize;
end;

procedure TMMJHintWindow.SetTextColor(Value:TColor);
begin
   if FTextColor<>Value then
      FTextColor:=Value;
   Invalidate;
end;

procedure TMMJHintWindow.SetFont(Value:TFont);
begin
   FHintFont.Assign(Value);
   if FHintFont <> nil then begin
      //Canvas.Font.Assign(Value);

      if  IsTrueTypeFont(Value) then begin
         // SetEnabled(True);
         Canvas.Font.Assign(Value);
         //Recaclulate the hint window
         CalcSize(Hint);
      end
      else begin
      //Change Font to Ariel if Not TTF selected
         FHintFont.Name:='Arial';
         Canvas.Font.Assign(FHintFont);
         Raise Exception.Create('Please select a TrueType font for Hint');
      end;
   end;
end;

//TMMJHint
procedure TMMJHint.SetFont(Value:TFont);
begin
   FFont.Assign(Value);
  // MMJHintWindow.CalcSize(Application.Hint)
  // TMMJHintWindow.SetFont(Value);
end;

constructor TMMJHint.Create(AOwner:TComponent);
begin
   inherited Create(AOwner);
   FTextColor:=clMaroon;
   FEnabled:=False;
   FFont:=TFont.Create;
   FFont.Name:='Arial';
   FShapeKind:=skEllipse;
   FStartColor:=clRed;
   FEndColor:=clGray;
   FFrameColor:=clDkGray;
   FBackgroundKind:=bkFill;
   FFillMode:=GradUpDown;
   FPicture := TPicture.Create;
   FHasFrame:=True;
end;

{destructor TMMJHint.Destroy;
begin
   FFont.Free;
   SetEnabled(False);
   inherited Destroy;
end; }

procedure TMMJHint.SetHasFrame(Value:Boolean);
begin
   if FHasFrame<> Value then
      FHasFrame:=Value;
end;

procedure TMMJHint.SetEnabled(Value:Boolean);
begin
   if FEnabled<> Value then
      FEnabled:=Value;

   //If Designing  Time
   if (csDesigning in ComponentState) then begin
      if FEnabled then
         FEnabled:=False
   end
   //if Run Time
   else begin
      if FEnabled then begin
      //reset application hint to new hint window
         Application.ShowHint:=False;
         HintWindowClass:=TMMJHintWindow;
         Application.ShowHint:=True;
         //Iterate through components of Application and set their SHowHint
         //Properties to true, otherwise error occurrs
      end;
      if not FEnabled then begin
         Application.ShowHint:=False;
         HintWindowClass:=THintWindow;
         Application.ShowHint:=True;
      end;
   end;
end;

procedure TMMJHint.SetTextColor(Value:TColor);
begin
   if FTextColor<>Value then
      FTextColor:=Value;
end;

procedure TMMJHint.SetShapeKind(Value : TShapeKind);
begin
   if FShapeKind<>Value then
      FShapeKind:=Value;
   //Invalidate;
end;

procedure TMMJHint.SetPicture(Value: TPicture);
begin
  FPicture.Assign(Value);
end;

procedure TMMJHint.SetStartColor(Value : TColor);
begin
   if FStartColor <> Value then
      FStartColor:=Value;
   //Invalidate;
end;

procedure TMMJHint.SetEndColor(Value : TColor);
begin
   if FEndColor <> Value then
      FEndColor:=Value;
  // Invalidate;
end;

procedure TMMJHint.SetBackgroundKind(Value:TBackgroundKind);
begin
   if FBackgroundKind <> Value then
      FBackgroundKind:=Value;
end;

procedure TMMJHint.SetFillMode(Value:TFillType);
begin
   if FFillMode <> Value then
      FFillMode:=Value;
end;

procedure TMMJHint.SetFrameColor(Value : TColor);
begin
   if FFrameColor<>Value then
      FFrameColor:=Value;
   //Invalidate;
end;

end.
